/**
 * @file
 * Zentropy javascript core
 *
 * - Initialize Zentropy namespace
 * - Provides frequently used functions:
 *   - Zentropy.trim(string)
 *   - Zentropy.indexOf(array, element, startFrom)
 *   - Zentropy.log(text)
 *   - Zentropy.imagePreload(image1, image2, ...)
 *   - Zentropy.msgBox(text)
 *   - Zentropy.ajaxStartLoad(block, text)
 *   - Zentropy.ajaxStopLoad(block, text)
 * - jQuery browser detection tweak
 */

// Initialize namespace
var Zentropy = Zentropy || {};

(function ($) {
  "use strict";

  /**
   * Trim a string
   * 
   * @param str string
   * @return String
   */
  Zentropy.trim = function (str) {
    return str.replace(/\s+$/, '').replace(/^\s+/, '');
  };

  /**
   * Find index of element in an array
   * Optionally starts from an index
   * Optionally use non-strict comparison
   * 
   * @param arr array
   * @param element mixed
   * @param startFrom int
   * @param strict bool
   */
  Zentropy.indexOf = function (arr, element, startFrom, strict) {
    var i, j;
    strict = strict || true;

    if (startFrom === null) {
      startFrom = 0;
    } else if (startFrom < 0) {
      startFrom = Math.max(0, arr.length + startFrom);
    }

    for (i = startFrom, j = arr.length; i < j; i += 1) {
      if (strict) {
        if (arr[i] === element) {
          return i;
        }
      } else {
        if (arr[i] == element) {
          return i;
        }
      }
    }

    return -1;
  };

  /**
   * Console.log wrapper to avoid errors when console is not present
   * usage: Zentropy.log('inside coolFunc', this, arguments);
   * 
   * @link http://paulirish.com/2009/log-a-lightweight-wrapper-for-consolelog/
   */
  Zentropy.log = function () {
    Zentropy.log.history = Zentropy.log.history || [];  // store logs to an array for reference
    Zentropy.log.history.push(arguments);
    if (window.console) {
      console.log(Array.prototype.slice.call(arguments));
    }
  };

  /**
   * Preloads one or more images
   */
  Zentropy.imagePreload = function () {
    var i, cacheImage = document.createElement('img');
    Zentropy.imagePreload.cache = Zentropy.imagePreload.cache || [];
    for (i = arguments.length; i > 0; i -= 1) {
      cacheImage.src = arguments[i];
      Zentropy.imagePreload.cache.push(cacheImage);
    }
  };

  /**
   * Displays a dropdown message
   * A duration can optionally be specified, if not the message stays on the screen for 2000 milliseconds
   * 
   * @param text string
   * @param duration int, in milliseconds
   */
  Zentropy.msgBox = function (text, duration) {
    duration = duration || 2000;
    Zentropy.ajaxStartLoad(false, text);

    setTimeout(function () {
      Zentropy.ajaxStopLoad(false);
    }, parseInt(duration, 10));
  };

  /**
   * Displays the Ajax Loader image
   * By default, the loader will block the User Interface
   * If a non-blocking loader is used, a message can optionally be displayed
   * 
   * @param block bool
   * @param msg string
   */
  Zentropy.ajaxStartLoad = function (block, msg) {
    var $loader, $body = $('body');
    block = typeof block !== 'undefined' ? block : true;

    if (block) {
      $loader = $body.find('#zentropy-loader');

      if (!$loader.length) {
        $loader = $('<div id="zentropy-loader" class="zentropy-loader" style="display:none;"><span class="zentropy-loader-image"></span></div>');
        $body.append($loader);
      }

      $loader.show();
    } else {
      $loader = $body.find('#zentropy-load-msg');

      if (!$loader.length) {
        msg = msg || Drupal.t('Please wait...');
        $loader = $('<div id="zentropy-load-msg" class="zentropy-load-msg" style="display:none;"><span class="zentropy-load-msg-text">' + msg + '</span></div>');
        $body.append($loader);
      }

      $loader.slideDown('fast');
    }
  };

  /**
   * Hides the Ajax Loader image
   * If a non-blocking loader is used, a message can optionally be displayed
   * 
   * @param block bool
   * @param msg string
   */
  Zentropy.ajaxStopLoad = function (block, msg) {
    var $loader, $body = $('body');
    block = typeof block !== 'undefined' ? block : true;
    msg = msg || null;

    if (block) {
      $loader = $body.find('#zentropy-loader');

      if ($loader.length) {
        $loader.remove();
      }
    } else {
      $loader = $body.find('#zentropy-load-msg');

      if ($loader.length) {
        if (msg) {
          $loader.find('span').text(msg);
          setTimeout(function () {
            $loader.slideUp('fast', function () {
              $(this).remove();
            });
          }, 1200);
        } else {
          $loader.slideUp('fast', function () {
            $(this).remove();
          });
        }
      }
    }
  };
}(jQuery));

// jQuery Browser Detect Tweak For IE7
jQuery.browser.version = jQuery.browser.msie && parseInt(jQuery.browser.version, 10) === 6 && window.XMLHttpRequest ? "7.0" : jQuery.browser.version;